function y = gtkef(x, m)
%implements generalized TKEF with variable lag m on signal x
% see W. Lin, C. Hamilton, and P. Chitrapu, A generalization to the teagerkaiser
% energy function and application to resolving two closely-spaced
% tones, in Proc. IEEE International Conference on Acoustics, Speech,
% and Signal Processing (ICASSP95), vol. 3, May 1995, pp. 16371640.

%x is 1-column vector, m is integer from 1 to < size(x)
%the first and last m values in y will be removed (not padded)

sz = size(x, 1);
sz;
y(1+m:sz-m) = x(1+m:sz-m) .^ 2;
y = y';
y(1:m) = 0;
y(sz-m+1:sz) = 0;
y(1+m:sz-m) = y(1+m:sz-m) - x(1:sz-2*m) .* x(1+2*m:sz);
y(1:m) = y(m+1); %pad first m values with y(m+1)
y(sz-m+1:sz) = y(sz-m); %pad last m values with y(sz-m)