function [out teagj pairs] = myfilters1(in, Fs, limits);
%implement a linear phase filter bank, log scale, for analysis using the
%Teager operator
%assume vector 'in' contains the input signal, sampled at Fs
%in should contain at least 300 samples, and frequencies should be in the
%  range specified by 'limits' below.
%out contains the output of all filters for all samples
%teagj contains the Teager operator value as applied to out
%pairs contains a set of {min, max} pairs corresponding to the band limits
%  of each filter

%Parts of this code are adapated from
%http://www.glue.umd.edu/~cknoll/enee739/project3.html, which provides a
%basic implementation of a linear phase filter bank in MATLAB

NF = 40; %num filters in bank 

lglims = log10(limits);
min = lglims(1);
max = lglims(2);
lgpairs = [(0:NF)-.5; (0:NF)+.5]/NF * max;
pairs = 10.^lgpairs; %pairs(:,i) gives band limits for filter i

N = 200; % filter order

clear B
%set B(:, i) to b vector for filter i
for i=1:NF
    B(:,i) = fir1(N, pairs(:,i)'./(Fs * .5))'; %note that this is changed from 
                                                %the code in http://www.glue.umd.edu/~cknoll/enee739/project3.html, 
                                                %as this contains a
                                                %mistake.
end

NS = length(in); % number of samples

clear out
%set out(:,i) to be output of filter i over the NS samples
for i=1:NF
    out(:,i) = filter(B(:,i),1,in(1:NS));
end

clear teagj;
for i = 1:NF
    teagj(:,i) = teager(out(:,i));
end

%display out and teagj by waterfall(out') or waterfall(teagj')
%warning: it takes a long time for MATLAB to generate the graph!
