function [w a fs as pairs out] = myfilters2(in, Fs, limits)
%implement a linear phase filter bank, log scale, for identification of
%amplitudes of component frquencies using DESA-2.
%assume vector 'in' contains the input signal, sampled at Fs
%in should contain at least 300 samples, and frequencies should be in the
%  range specified by 'limits' below.
%pairs and out are the same as returned by myfilters1.
%w and a return the frequency and amplitude values for the signal component
%falling within each filter, with the exception that w and a are zeroed out for 
%all filters i where w is not in i's frequency band.
%fs and as are corresponding frequency and amplitude pairs for signal
%components, averaged over the size of the input


%Parts of this code are adapated from
%http://www.glue.umd.edu/~cknoll/enee739/project3.html, which provides a
%basic implementation of a linear phase filter bank in MATLAB

NF = 50; %num filters in bank

lglims = log10(limits);
min = lglims(1);
max = lglims(2);
lgpairs = [(0:NF)-.5; (0:NF)+.5]/NF * (max-min) + min;
pairs = 10.^lgpairs; %pairs(:,i) gives band limits for filter i

N = 200; % filter order

clear B
 %set B(:, i) to b vector for filter i
 for i=1:NF
     B(:,i) = fir1(N, pairs(:,i)'./(Fs * .5))'; %note that this is changed from 
                                                %the code in http://www.glue.umd.edu/~cknoll/enee739/project3.html, 
                                                %as this contains a mistake.
 end

NS = length(in); % number of samples

clear out
%set out(:,i) to be output of filter i over the NS samples
for i=1:NF
    out(:,i) = filter(B(:,i),1,in(1:NS));
end


clear w; 
clear a;
for i=1:NF
    [w(:,i) a(:,i)] = desa2(out(:,i));
end

%get rid of imaginary parts of a and w; these are errors
w = real(w);
a = real(a);

%analyze signal 200 signals in until end to discount effects of initial
%delay
w2 = w(201:end,:);
a2 = a(201:end,:);

wmeans = mean(w2(:,:));

%convert wmeans to frequencies in Hz, and find correspdonding filters
freqs = wmeans * Fs/ (2 * pi);
freqs = unique(ceil(freqs));

filts = unique(find_filt(freqs, NF, limits)); %set of filters containing component frequencies

for i = 1:NF
     c = size(intersect(i, filts));
     if c(1) ~= 1
         a(:,i) = zeros(size(a(:,i)));
         w(:,i) = zeros(size(w(:,i)));
     end
end

clear fs as;
count = 1;
for i = 1:NF
    if ~(isequal(intersect(i, filts), []))
        wi = mean(w(201:end,i));
        fi = Fs * wi / (2 * pi);
        fs(count) = fi;
        ai = mean(a(201:end,i));
        as(count) = ai;
        count = count+1;
    end
end

%as and fs can now be displayed easily using stem
