%Part 1: Demonstrate basic teager functionality with basic signal inputs,
%with and without noise.

t = 0:1/8000:500/8000;
t = t';
Fs = 8000; %8000 Hz sample rate

%a. generate sine, damped sine, chirp, and two-component signals, and
%calculate TKEO.

sin1 = sin (t * Fs * pi / 4);
tsin1 = teager(sin1);

damped_sin = 2 * exp(-.05*t*Fs) .* sin1;
tdamped_sin = teager(damped_sin);

ch1 = chirp(t, (pi/4 * Fs/(2 * pi)), 125/Fs, 0);
clear ch;
ch(1:125) = ch1(1:125);
ch(126:250) = ch1(125:-1:1);
ch = ch';
tch = teager(ch);

sin2 = sin(t * Fs * pi/4) + sin(t * Fs * pi/6);
tsin2 = teager(sin2);


%b. generate noisy versions of these signals and show TKEO
SNR = 50;

sin1_noisy = gen_noise(sin1, SNR);
tsin1_noisy = teager(sin1_noisy);

damped_sin_noisy = gen_noise(damped_sin, SNR);
tdamped_sin_noisy = teager(damped_sin_noisy);

ch_noisy = gen_noise(ch, SNR);
tch_noisy = teager(ch_noisy);

sin2_noisy = gen_noise(sin2, SNR);
tsin2_noisy = teager(sin2_noisy);

%c. generate noisy versions of one signal with variable noise
SNR1 = 200;
SNR2 =  100;
SNR3 = 50;
SNR4 =  25;

sin_n1 = gen_noise(sin1, SNR1);
tsin_n1 = teager(sin_n1);

sin_n2 = gen_noise(sin1, SNR2);
tsin_n2 = teager(sin_n2);

sin_n3 = gen_noise(sin1, SNR3);
tsin_n3 = teager(sin_n3);

sin_n4 = gen_noise(sin1, SNR4);
tsin_n4 = teager(sin_n4);

%d. Show plots

%%basic signals, noisy signals, and TKEO:
a = figure;
set(a, 'name', 'Teager on Sine and Damped Sine Waves', 'NumberTitle', 'off');
subplot(4,2,1);
plot(sin1(1:50));
title('simple sine wave and TKEO');
ylim([-1.5 1.5]);
subplot(4,2,3);
plot(tsin1(1:50));
ylim([0 1]);

subplot(4,2,2);
plot(sin1_noisy(1:50));
title('noisy simple sine wave and TKEO');
ylim([-1.5 1.5]);
subplot(4,2,4);
plot(tsin1_noisy(1:50));
ylim([0 1]);

subplot(4,2,5);
plot(damped_sin(1:50));
title('damped sine wave and TKEO');
ylim([-2 2]);
subplot(4,2,7);
plot(tdamped_sin(1:50));
ylim([0 2]);

subplot(4,2,6);
plot(damped_sin_noisy(1:50));
title('noisy damped sine wave and TKEO');
ylim([-2 2]);
subplot(4,2,8);
plot(tdamped_sin_noisy(1:50));
ylim([0 2]);

a = figure;
set(a, 'name', 'Teager on Chirp and 2-Component Signals', 'NumberTitle', 'off');
subplot(4,2,1);
plot(ch);
title('chirp and TKEO');
ylim([-1.5 1.5]);
subplot(4,2,3);
plot(tch);
ylim([-.5 .75]);

subplot(4,2,2);
plot(ch_noisy);
title('noisy chirp and TKEO');
ylim([-1.5 1.5]);
subplot(4,2,4);
plot(tch_noisy);
ylim([-.5 .75]);

subplot(4,2,5);
plot(sin2(1:100));
title('composite signal and TKEO');
ylim([-2.5 2.5]);
subplot(4,2,7);
plot(tsin2(1:100));
ylim([0 2.5]);

subplot(4,2,6);
plot(sin2_noisy(1:100));
title('noisy composite signal and TKEO');
ylim([-2.5 2.5]);
subplot(4,2,8);
plot(tsin2_noisy(1:100));
ylim([0 2.5]);


%%show one signal with variable noise
a = figure;
set(a, 'name', 'Effect of Noise on the Teager Operator', 'NumberTitle', 'off');
subplot(5,2,1);
plot(sin1(1:50));
title('noiseless sine wave');
ylim([-2 2]);
subplot(5,2,2);
plot(tsin1(1:50));
title('TKEO on noiseless sine wave');
ylim([0 2]);


subplot(5,2,3);
plot(sin_n1(1:50));
ttl1 = 'sine wave with SNR = ';
title([ttl1 num2str(SNR1)]);
ylim([-2 2]);
subplot(5,2,4);
plot(tsin_n1(1:50));
ttl2 = 'TKEO on sin wave with SNR = ';
title([ttl2 num2str(SNR1)]);
ylim([0 2]);

subplot(5,2,5);
plot(sin_n2(1:50));
ttl1 = 'sine wave with SNR = ';
ylim([-2 2]);
title([ttl1 num2str(SNR2)]);
subplot(5,2,6);
plot(tsin_n2(1:50));
ttl2 = 'TKEO on sin wave with SNR = ';
title([ttl2 num2str(SNR2)]);
ylim([0 2]);

subplot(5,2,7);
plot(sin_n3(1:50));
ttl1 = 'sine wave with SNR = ';
ylim([-2 2]);
title([ttl1 num2str(SNR3)]);
subplot(5,2,8);
plot(tsin_n3(1:50));
ttl2 = 'TKEO on sin wave with SNR = ';
title([ttl2 num2str(SNR3)]);
ylim([0 2]);

subplot(5,2,9);
plot(sin_n4(1:50));
ttl1 = 'sine wave with SNR = ';
ylim([-2 2]);
title([ttl1 num2str(SNR4)]);
subplot(5,2,10);
plot(tsin_n4(1:50));
ttl2 = 'TKEO on sin wave with SNR = ';
title([ttl2 num2str(SNR4)]);
ylim([0 2]);
