%Part 2: Examine performance of DESA-2 on different inputs, with and
%without noise.
% For an explanation of DESA-2, see P. Maragos, J. F. Kaiser, and T. F. Quatieri, On separating amplitude
% from frequency modulations using energy operators, in Proc. IEEE
% International Conference on Acoustics, Speech, and Signal Processing
% (ICASSP92), vol. 2, Mar. 1992, pp. 14.


t = 0:1/8000:500/8000;
t = t';
Fs = 8000;

%a. Show DESA-2 in calculating frequency and amplitude for regular sine,
%damped sine, and chirp.

sin1 = sin (t * Fs * pi / 4);
sin1w = ones(500,1) * (Fs * (pi/4) / (2*pi));
sin1a = ones(500,1);
[sin1dw sin1da] = desa2(sin1);
sin1dw = sin1dw * Fs / (2 * pi); %convert from rad/sample to Hz

damped_sin = 2 * exp(-.05*t*Fs) .* sin1;
damped_sinw = sin1w;
damped_sina = 2 * exp(-.05*t*Fs);
[damped_sindw damped_sinda] = desa2(damped_sin);
damped_sindw = damped_sindw * Fs / (2 * pi);

clear ch ch1;
ch1 = chirp(t, (pi/4 * Fs/(2 * pi)), 125/Fs, 0);
ch(1:125) = ch1(1:125);
ch(126:250) = ch1(125:-1:1);
ch = ch';
cha = ones(1,250);
chw = (pi/4 * Fs / (2 * pi)):-1*(pi/4 * Fs / (2 * pi))/125:0;
chw(126:251) = 0:(pi/4 * Fs /(2*pi))/125:(pi/4*Fs / (2* pi));
chw(251) = [];
[chdw chda] = desa2(ch);
chdw = real(chdw);
chda = real(chda);
chdw = chdw * Fs / (2 * pi);

%b. Look at error for sine at different frequencies

freq1 = (2 * pi / 20) ;
freq2 = (4 * pi / 20) ;
freq3 = (5 * pi / 20) ;
freq4 = (8 * pi / 20);
freq5 = (10 * pi / 20);

sinf1 = sin (t * Fs * freq1);
sinf2 = sin (t * Fs * freq2);
sinf3 = sin (t * Fs * freq3);
sinf4 = sin (t * Fs * freq4);
sinf5 = sin (t * Fs * freq5);

[sinf1dw sinf1da] = desa2(sinf1);
[sinf2dw sinf2da] = desa2(sinf2);
[sinf3dw sinf3da] = desa2(sinf3);
[sinf4dw sinf4da] = desa2(sinf4);
[sinf5dw sinf5da] = desa2(sinf5);

sinf1dw = real(sinf1dw);
sinf1da = real(sinf1da);
sinf2dw = real(sinf2dw);
sinf2da = real(sinf2da);
sinf3dw = real(sinf3dw);
sinf3da = real(sinf3da);
sinf4dw = real(sinf4dw);
sinf4da = real(sinf4da);
sinf5dw = real(sinf5dw);
sinf5da = real(sinf5da);

err1w = abs((sinf1dw(3:498) - freq1))/freq1;
err2w = abs((sinf2dw(3:498) - freq2))/freq2;
err3w = abs((sinf3dw(3:498) - freq3))/freq3;
err4w = abs((sinf4dw(3:498) - freq4))/freq4;
err5w = abs((sinf5dw(3:498) - freq5))/freq5;

err1a = abs((sinf1da(3:498) - 1))/1;
err2a = abs((sinf2da(3:498) - 1))/1;
err3a = abs((sinf3da(3:498) - 1))/1;
err4a = abs((sinf4da(3:498) - 1))/1;
err5a = abs((sinf5da(3:498) - 1))/1;


meanerr1w = [ mean(err1w) mean(err2w) mean(err3w) mean(err4w) mean(err5w)]';
meanerr1w = meanerr1w * 100;
meanerr1a = [ mean(err1a) mean(err2a) mean(err3a) mean(err4a) mean(err5a)]';
meanerr1a = meanerr1a * 100;
freqs = [freq1 freq2 freq3 freq4 freq5]';

%c. look at error for damped sine for different frequencies

damped_sinf1 = 2 * exp(-.05*t*Fs) .* sinf1;
damped_sinf2 = 2 * exp(-.05*t*Fs) .* sinf2;
damped_sinf3 = 2 * exp(-.05*t*Fs) .* sinf3;
damped_sinf4 = 2 * exp(-.05*t*Fs) .* sinf4;
damped_sinf5 = 2 * exp(-.05*t*Fs) .* sinf5;

[d_sinf1dw d_sinf1da] = desa2(damped_sinf1);
[d_sinf2dw d_sinf2da] = desa2(damped_sinf2);
[d_sinf3dw d_sinf3da] = desa2(damped_sinf3);
[d_sinf4dw d_sinf4da] = desa2(damped_sinf4);
[d_sinf5dw d_sinf5da] = desa2(damped_sinf5);

d_sinf1dw = real(d_sinf1dw);
d_sinf1da = real(d_sinf1da);
d_sinf2dw = real(d_sinf2dw);
d_sinf2da = real(d_sinf2da);
d_sinf3dw = real(d_sinf3dw);
d_sinf3da = real(d_sinf3da);
d_sinf4dw = real(d_sinf4dw);
d_sinf4da = real(d_sinf4da);
d_sinf5dw = real(d_sinf5dw);
d_sinf5da = real(d_sinf5da);

d_err1w = abs((d_sinf1dw(3:498) - freq1))/freq1;
d_err2w = abs((d_sinf2dw(3:498) - freq2))/freq2;
d_err3w = abs((d_sinf3dw(3:498) - freq3))/freq3;
d_err4w = abs((d_sinf4dw(3:498) - freq4))/freq4;
d_err5w = abs((d_sinf5dw(3:498) - freq5))/freq5;

d_err1a = abs((d_sinf1da(3:498) - damped_sina(3:498)))./damped_sina(3:498);
d_err2a = abs((d_sinf2da(3:498) - damped_sina(3:498)))./damped_sina(3:498);
d_err3a = abs((d_sinf3da(3:498) - damped_sina(3:498)))./damped_sina(3:498);
d_err4a = abs((d_sinf4da(3:498) - damped_sina(3:498)))./damped_sina(3:498);
d_err5a = abs((d_sinf5da(3:498) - damped_sina(3:498)))./damped_sina(3:498);


d_meanerr1w = [ mean(d_err1w) mean(d_err2w) mean(d_err3w) mean(d_err4w) mean(d_err5w)]';
d_meanerr1w = d_meanerr1w * 100;
d_meanerr1a = [ mean(d_err1a) mean(d_err2a) mean(d_err3a) mean(d_err4a) mean(d_err5a)]';
d_meanerr1a = d_meanerr1a * 100;


%d. Demonstrate effect of noise on DESA-2 performance
SNR1 = 100;

sin1_noisy = gen_noise(sin1, SNR1);

[sin1_noisyw sin1_noisya] = desa2(sin1_noisy);

sin1_noisyw = real(sin1_noisyw);
sin1_noisya = real(sin1_noisya);

sin1_noisywerr = (sin1_noisyw - freq1)/freq1 * 100;
sin1_noisyaerr = (sin1_noisya - 1)/1 * 100;

display( sprintf('Mean percentage error for DESA-2 frequency when SNR = %d is %f.', SNR1, mean(sin1_noisywerr)));
display( sprintf('Mean percentage error for DESA-2 amplitude when SNR = %d is %f.', SNR1, mean(sin1_noisyaerr)));

%%Display graphs
a = figure;
set(a, 'name', 'Real and DESA-2 Amplitude and Frequency for Simple Sine', 'NumberTitle', 'off');
subplot(3,2,1);
plot(sin1(1:50));
title('Simple sine signal');
subplot(3,2,3);
plot(sin1a(1:50));
title('Amplitude');
ylim([0 1.5]);
subplot(3,2,4);
plot(sin1w(1:50));
title('Frequency (rads/sample)');
ylim([998 1001]);
subplot(3,2,5);
plot(sin1da(1:50));
title('DESA-2 Amplitude');
ylim([0 1.5]);
subplot(3,2,6);
plot(sin1dw(1:50));
title('DESA-2 Frequency');
ylim([998 1001]);

a = figure;
set(a, 'name', 'Real and DESA-2 Amplitude and Frequency for Damped Sine', 'NumberTitle', 'off');
subplot(3,2,1);
plot(damped_sin(1:50));
title('Damped sine signal');
subplot(3,2,3);
plot(damped_sina(1:50));
title('Amplitude');
ylim([0 2]);
subplot(3,2,4);
plot(damped_sinw(1:50));
ylim([998 1008]);
title('Frequency');

subplot(3,2,5);
plot(damped_sinda(1:50));
title('DESA-2 Amplitude');
ylim([0 2]);
subplot(3,2,6);
damped_sindw = real(damped_sindw);
plot(damped_sindw(1:50));
title('DESA-2 Frequency');
ylim([998 1008]);

a = figure;
set(a, 'name', 'Real and DESA-2 Amplitude and Frequency for Chirp', 'NumberTitle', 'off');
subplot(3,2,1);
plot(ch);
title('Chirp signal');
subplot(3,2,3);
plot(cha);
title('Amplitude (*see paper)');
ylim([0 2]);
subplot(3,2,4);
plot(chw);
title('Frequency');
ylim([0 1000]);
subplot(3,2,5);
plot(chda(1:250));
title('DESA-2 Amplitude');
ylim([0 2]);
subplot(3,2,6);
plot(chdw(1:250));
title('DESA-2 Frequency');
ylim([0 1000]);

a = figure;
set(a, 'name', '% Error for DESA-2 Amplitude and Frequency, Damped and Simple Sine', 'NumberTitle', 'off');
subplot(2,2,1);
plot(freqs,meanerr1w,'-r',freqs,meanerr1w,'ok');
title('simple sine, DESA-2 frequency % error')
subplot(2,2,2);
plot(freqs,meanerr1a,'-r',freqs,meanerr1a,'ok');
title('simple sine, DESA-2 amplitude % error')
subplot(2,2,3);
plot(freqs,d_meanerr1w,'-r',freqs,d_meanerr1w,'ok');
title('damped sine, DESA-2 frequency % error')
subplot(2,2,4);
plot(freqs,d_meanerr1a,'-r',freqs,d_meanerr1a,'ok');
title('damped sine, DESA-2 amplitude % error')


a = figure;
set(a, 'name', 'Effect of Noise on DESA-2', 'NumberTitle', 'off');
subplot(2,2,1);
plot(sin1_noisy(1:50));
title(['Sin wave with SNR = ' num2str(SNR1)]);
subplot(2,2,3);
plot(sin1_noisyw(1:50));
title('DESA-2 frequency');
subplot(2,2,4);
plot(sin1_noisya(1:50));
title('DESA-2 amplitude');

