%Part 3: Demonstrate generalized TKEO in resolving two tones
%based on W. Lin, C. Hamilton, and P. Chitrapu, A generalization to the teagerkaiser
% energy function and application to resolving two closely-spaced
% tones, in Proc. IEEE International Conference on Acoustics, Speech,
% and Signal Processing (ICASSP95), vol. 3, May 1995, pp. 16371640.

%note: spec plots displayed by this script can be resized for better
%resolution; 0-1000 works well for X-axis.

%a. Create a signal of two closely spaced tones.
t = 0:1/8000:1;
t = t';
Fs = 8000;

sin1 = sin(2 * pi * t * 330) + sin (2 * pi * t * 300);

%b. Show the original TKEO on two tones, plus its FFT
tkeo = teager(sin1);
a = figure;
set(a, 'name', 'TKEO on noiseless two-component signal', 'NumberTitle', 'off');
plot(tkeo(1:1000));
a = figure;
set(a, 'name', 'FFT of TKEO on noiseless two-component signal', 'NumberTitle', 'off');
disp_spec2(tkeo, Fs);

%c. show the original TKEO on two noisy tones, plus its FFT
sin1_noisy = gen_noise(sin1, 25);
tkeo_noisy = teager(sin1_noisy);
a = figure;
set(a, 'name', 'TKEO on noisy two-component signal, SNR=25', 'NumberTitle', 'off');
plot(tkeo_noisy(1:1000));
a = figure;
set(a, 'name', 'FFT of TKEO on noisy two-component signal, SNR=25', 'NumberTitle', 'off');
disp_spec2(tkeo_noisy, Fs);

%d. Show the GTKEF on two noiseless tones, with m = 25
%   Repeat with noise.
gtkeo25 = gtkef(sin1, 25);
a = figure;
set(a, 'name', 'GTKEO on noiselss signal, m=25', 'NumberTitle', 'off');
plot(gtkeo25(1:1000));
 a = figure;
set(a, 'name', 'FFT of GTKEO on noiselss signal, m=25', 'NumberTitle', 'off');
disp_spec2(gtkeo25, Fs);

gtkeo25_noisy = gtkef(sin1_noisy, 25);
a = figure;
set(a, 'name', 'GTKEO on noisy signal, SNR=25, m=25', 'NumberTitle', 'off');
plot(gtkeo25_noisy(1:1000));
a = figure;
set(a, 'name', 'FFT of GTKEO on noisy signal, SNR=25, m=25', 'NumberTitle', 'off');
disp_spec2(gtkeo25_noisy, Fs);


%e. Show the GTKEF on two noiseless tones, with m = 6
%   Repeat with noise.
gtkeo6 = gtkef(sin1, 6);
a = figure;
set(a, 'name', 'GTKEO on noiselss signal, m=6', 'NumberTitle', 'off');
plot(gtkeo6(1:1000));
 a = figure;
set(a, 'name', 'FFT of GTKEO on noiselss signal, m=6', 'NumberTitle', 'off');
disp_spec2(gtkeo6, Fs);

gtkeo6_noisy = gtkef(sin1_noisy, 6);
a = figure;
set(a, 'name', 'GTKEO on noisy signal, SNR=25, m=6', 'NumberTitle', 'off');
plot(gtkeo6_noisy(1:1000));
a = figure;
set(a, 'name', 'FFT of GTKEO on noisy signal, SNR=25, m=6', 'NumberTitle', 'off');
disp_spec2(gtkeo6_noisy, Fs);


